#ifndef _COREINTERFACE_H
#define _COREINTERFACE_H

class CoreInterface; // forward declaratoin needed because of C++ choobiness

#include "../include/securityinterface.h"
#include "../include/configinterface.h"
#include "../include/plugininterface.h"
#include "../include/commandinterface.h"

#include <QDomNode>
#include <QHash>
#include <QLibrary>
#include <QList>
#include <QObject>
#include <QString>
#include <QTcpSocket>

class CoreInterface : public QObject
{
Q_OBJECT
public:
	CoreInterface(); // load plugins etc
	virtual QString lastError() { return _lastError; };
	virtual SecurityInterface* security(){ return _security; };
	virtual void loopbackMessage(QString to, QString from, QString message);
	virtual void reply(QString to, QString from, bool pm, QString message);
	virtual void sendMessage(QString to, QString message);
	virtual void sendAction(QString to, QString message);
	virtual void sendNotice(QString to, QString message);
	virtual void sendRaw(QString command);
	virtual void joinChannel(QString channel);
	virtual void partChannel(QString channel);
	virtual void unregisterPlugin(PluginInterface* plugin);
	virtual void registerCommand(PluginInterface* plugin, QString command, CommandInterface* commandObject);
	
	QString ircd(){return _ircd; };
	
	QHash<QString,PluginInterface*> pluginList() { return _pluginObjects; };
  QHash<PluginInterface*, QHash<QString, CommandInterface*> > commands; // _commands[plugin]['command']
	bool loadPlugin(QString name);
	bool unloadPlugin(QString name, bool isReloading = false); // please don't lie...
	bool reloadPlugin(QString name);
	QString getConfigValue(QString key);
	QDomDocument configDoc();
	QDomElement getPluginConfig(QString pluginName);
	void saveConfig();
private slots:
	void slotReadData();
	void slotStateChanged(QAbstractSocket::SocketState newState);
	void slotError(QAbstractSocket::SocketError socketError);
private:
	QHash<QString, QLibrary*> _pluginLibraryHandles;
	QHash<QString, PluginInterface*> _pluginObjects;
	QList<PluginInterface*> _filters;
	QString _ircd;
	QString _lastError;
	SecurityInterface* _security;
	QTcpSocket* _socket;
	ConfigInterface* _config;
	QList<PluginInterface*> _deletedPlugins; // in this message
	QList<PluginInterface*> _toDeletePlugins; // for after this message
	QList<QLibrary*> _toCloseLibraries;
	QStringList _toReloadPlugins; // for after this message
};

#endif
