#include "Aliases.h"

#include <iostream>
using namespace std;

#include <QRegExp>
#include <QUrl>

extern "C" PluginInterface* getObject(CoreInterface* core)
{
	return (PluginInterface*) new Aliases(core);
}

QString Aliases::title()
{
	return "Aliases";
}

QString Aliases::description()
{
	return "Allows you to assign short names for commands";
}

Aliases::Aliases(CoreInterface* core) : PluginInterface()
{
	_core = core;
	QDomElement configRoot = _core->getPluginConfig("aliases");

	for (QDomNode i = configRoot.firstChild(); !i.isNull(); i = i.nextSibling())
	{
		if ( i.nodeName() == "alias" )
		{
			QString shortCmd = "";
			QString fullCmd = "";

			for (QDomNode j = i.firstChild(); !j.isNull(); j = j.nextSibling())
			{
				if ( j.nodeName() == "short" )
					shortCmd = j.firstChild().nodeValue();
				if ( j.nodeName() == "full" )
					fullCmd = j.firstChild().nodeValue();
			}
			if ( shortCmd != "" && fullCmd != "" )
				_aliases[shortCmd] = fullCmd;
		}
	}
	
	_core->registerCommand(this, "add", new AliasesCommands::add(_core, &_aliases));
	_core->registerCommand(this, "show", new AliasesCommands::show(_core, &_aliases));
	_core->registerCommand(this, "remove", new AliasesCommands::remove(_core, &_aliases));
	_core->registerCommand(this, "list", new AliasesCommands::list(_core, &_aliases));
	_core->registerCommand(this, "_message", new AliasesCommands::_message(_core, &_aliases));
}

QStringList Aliases::helpText()
{
	return QStringList("The aliases plugin allows you to assign a shorter name for a command.");
}
