#include "../aliasescommands.h"

namespace AliasesCommands
{
	add::add(CoreInterface* core, QHash<QString,QString>* aliases)
	{
		_core = core;
		_aliases = aliases;
	}
	
	QString add::description()
	{
		return "Adds an alias";
	}
	
	QStringList add::syntax()
	{
		return QStringList("<shortname> <command>");
	}
	
	QStringList add::helpText()
	{
		QStringList output;
		output 
			<< "Shortname is a single word, command may be any number of words, starting with a bot command."
			<< "After adding an alias, typing shortname is equivalent to typing command."
			<< "command may contain the following variables:"
	    << " $window     : replaced with the name of the active window (channel name or msg from)"
      << " $from       : replaced with the name of the person calling the alias"
   	  << " $n          : replaced with the Nth word following shortname"
   	  << " $params     : replaced with all text following sortname"
   	  << " $httpparams : replaced with http-escaped version of $params";
		return output;
	}
  
	void add::go(QString to, QString from, bool pm, QString params)
	{
		QStringList words = params.split(" ");
    if ( words.count() >= 2 )
    {
      QString shortCmd = words.takeFirst().toLower();
      QString fullCmd = words.join(" ");

      if ( _aliases->contains(shortCmd) )
      {
        _core->reply(to, from, pm, QString("'%1' is already used for an alias.").arg(shortCmd));
        return;
      }

      if ( shortCmd.contains(".") )
      {
        _core->reply(to, from, pm, QString("ShortCommands can not contain '.'."));
        return;
      }
	
      (*_aliases)[shortCmd] = fullCmd;

      // Save to XML
      QDomElement configRoot = _core->getPluginConfig("aliases");

      QDomElement newAlias = _core->configDoc().createElement("alias");
      configRoot.appendChild(newAlias);

        QDomElement newShortTag = _core->configDoc().createElement("short");
        newAlias.appendChild(newShortTag);

          QDomText newShortText = _core->configDoc().createTextNode(shortCmd);
          newShortTag.appendChild(newShortText);

        QDomElement newFullTag = _core->configDoc().createElement("full");
        newAlias.appendChild(newFullTag);

          QDomText newFullText = _core->configDoc().createTextNode(fullCmd);
          newFullTag.appendChild(newFullText);

      _core->saveConfig();
      _core->reply(to, from, pm, "Alias added.");
    }
    else
      _core->reply(to, from, pm, "Syntax: Aliases.add <shortname> <command>");
	}
};
