#include "../../include/coreinterface.h"
#include "configplugin.h"

#include <iostream>
#include <string>
using namespace std;

#include <QDomDocument>
#include <QFile>
#include <QString>
#include <QTextStream>

extern "C" PluginInterface* getObject(CoreInterface* Core)
{
	return new ConfigPlugin();
}

ConfigPlugin::ConfigPlugin()
{
	currentConfig = 0;
	loadConfig();
}

QString ConfigPlugin::title()
{
	return "Config";
}

QString ConfigPlugin::description()
{
	return "An XML-based configuration system";
}

QString ConfigPlugin::getConfigValue(QString key)
{
	QDomElement root = currentConfig->documentElement();
	for (QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling())
		if (i.nodeName() == "core")
			for (QDomNode j = i.firstChild(); !j.isNull(); j = j.nextSibling())
				if ( j.nodeName() == key ) return j.firstChild().nodeValue();
	return "";
}

void ConfigPlugin::setConfigValue(QString key, QString value)
{
  QDomElement root = currentConfig->documentElement();
	QDomNode i;
  for (i = root.firstChild(); !i.isNull(); i = i.nextSibling())
    if (i.nodeName() == "core")
		{
      for (QDomNode j = i.firstChild(); !j.isNull(); j = j.nextSibling())
        if ( j.nodeName() == key )
				{
					j.firstChild().setNodeValue(value);
					return;
				}
			break;
		}
	// If we get here, the value has not currently been set
	QDomElement valueTag = currentConfig->createElement(key);
	i.appendChild(valueTag);

	QDomText valueText = currentConfig->createTextNode(value);
	valueTag.appendChild(valueText);

	saveConfig();
}

QDomElement ConfigPlugin::getPluginNode(QString plugin)
{
	QDomNodeList plugins = currentConfig->elementsByTagName("plugin");
	for (int i = 0; i < plugins.length(); i++)
	{
		if (plugins.item(i).attributes().namedItem("name").nodeValue() == plugin)
			return plugins.item(i).toElement();
	}
	return *(new QDomElement());
}

QDomDocument ConfigPlugin::getDoc()
{
	return *currentConfig;
}

void ConfigPlugin::loadPlugin(QString name)
{
	if ( ! currentConfig ) return;

	if ( ! getFullPluginList().contains(name) )
	{
		QDomElement pluginElement = currentConfig->createElement("plugin");
		pluginElement.setAttribute("loaded","true");
		pluginElement.setAttribute("name", name);
		currentConfig->documentElement().appendChild(pluginElement);
	}
	else
	{
		QDomNode pluginNode = getPluginNode(name);
		pluginNode.attributes().namedItem("loaded").setNodeValue("true"); 	
	}
	saveConfig();
}

void ConfigPlugin::unloadPlugin(QString name)
{
	if ( getFullPluginList().contains(name) )
	{
		QDomNode pluginNode = getPluginNode(name);
		pluginNode.attributes().namedItem("loaded").setNodeValue("false");
	}
	saveConfig();
}

QStringList ConfigPlugin::getPluginList()
{
	if ( ! currentConfig ) return QStringList();
	QStringList retVal;
	QDomNodeList plugins = currentConfig->elementsByTagName("plugin");
	for (int i = 0; i < plugins.length(); i++)
	{
		if (plugins.item(i).attributes().namedItem("loaded").nodeValue() == "true")
			retVal.append(plugins.item(i).attributes().namedItem("name").nodeValue());
	}
	return retVal;
}

QStringList ConfigPlugin::getFullPluginList()
{
	if ( ! currentConfig ) return QStringList();
  QStringList retVal;
  QDomNodeList plugins = currentConfig->elementsByTagName("plugin");
  for (int i = 0; i < plugins.length(); i++)
    retVal.append(plugins.item(i).attributes().namedItem("name").nodeValue());
  return retVal;
}
void ConfigPlugin::loadConfig()
{
	if ( currentConfig ) delete currentConfig;
	QFile file("MKbot.rc");
	if ( ! file.open(QIODevice::ReadOnly | QIODevice::Text) )
	{
		createConfig();
		return;
	}

	currentConfig = new QDomDocument("MKbot");
	currentConfig->setContent(&file);
	file.close();
}

void ConfigPlugin::saveConfig()
{
	QFile file("MKbot.rc");
	if ( ! file.open(QIODevice::WriteOnly | QIODevice::Text) )
	{
		cerr << "Could not open MKbot.rc for writing.";
		return;
	}
	QTextStream out(&file);
	out << currentConfig->toString();
	file.close();
}

void ConfigPlugin::createConfig()
{
	string serverName, serverPort, botNick, commandPrefix;
	cout << "Enter a nickname for the bot: ";
	cin >> botNick;
	cout << "Enter the server hostname to connect to: ";
	cin >> serverName;
	cout << "Enter the server port to connect to: ";
	cin >> serverPort;
	cout << "Enter the command prefix: ";
	cin >> commandPrefix;

	if ( currentConfig ) delete currentConfig;
	currentConfig = new QDomDocument("MKbot");
	QDomElement root = currentConfig->createElement("config");
	currentConfig->appendChild(root);

	QDomElement core = currentConfig->createElement("core");
	root.appendChild(core);
	
	QDomElement nickTag = currentConfig->createElement("nick");
	core.appendChild(nickTag);

	QDomText nickText = currentConfig->createTextNode(QString::fromStdString(botNick));
	nickTag.appendChild(nickText);

	QDomElement serverTag = currentConfig->createElement("server");
	core.appendChild(serverTag);

	QDomText serverText = currentConfig->createTextNode(QString::fromStdString(serverName));
	serverTag.appendChild(serverText);

	QDomElement portTag = currentConfig->createElement("port");
	core.appendChild(portTag);

	QDomText portText = currentConfig->createTextNode(QString::fromStdString(serverPort));
	portTag.appendChild(portText);

	QDomElement prefixTag = currentConfig->createElement("prefix");
	core.appendChild(prefixTag);
	
	QDomText prefixText = currentConfig->createTextNode(QString::fromStdString(commandPrefix));
	prefixTag.appendChild(prefixText);

	QDomElement pluginLoaderPlugin = currentConfig->createElement("plugin");
	pluginLoaderPlugin.setAttribute("name", "pluginLoader");
	pluginLoaderPlugin.setAttribute("loaded", "true");
	root.appendChild(pluginLoaderPlugin);

	saveConfig();
}
