#include "../helpcommands.h"

namespace HelpCommands
{
	help::help(CoreInterface* core)
	{
		_core = core;
	}
	
	QString help::description()
	{
		return "Retrieves help for MKbot functionality";
	};
	
	QStringList help::syntax()
	{
		QStringList output;
		output
			<< ""
			<< "<plugin>"
			<< "<plugin>.<command>";
		return output;
	}
	
	QStringList help::helpText()
	{
		QStringList output;
		output
			<< "This command will either show help on the help system if no arguments are given, otherwise it will give help on the specified plugin and/or command.";
		return output;
	}
	
	void help::go(QString to, QString from, bool pm, QString params)
	{
		if ( ! pm ) _core->reply(to, from, pm, QString("%1: Sending in private message").arg(from));

		if ( params == "" )
		{
			QStringList output;
			output
				<< "Help"
				<< "----"
				<< "Syntax:"
				<< "> Help.Help"
				<< "> Help.Help <plugin>"
				<< "> Help.Help <plugin> <command"
				<< "----"
				<< "Loaded plugins:";
			foreach(PluginInterface* obj, _core->pluginList())
				output << QString("%1 : %2").arg(obj->title(), -16).arg(obj->description());
			output << "----";
			foreach(QString line, output)
				_core->sendMessage(from, line);
			return;
		}
			
		QStringList words = params.split(".");
		if ( words.count() > 2 )
		{
			_core->sendMessage(from, "Invalid syntax; See Help.Help");
			return;
		}

		QString plugin = words[0].toLower();
		if ( ! _core->pluginList().contains(plugin) )
		{
			_core->sendMessage(from, QString("No such plugin: '%1'").arg(words[0]));
			return;
		}
			
		if ( words.count() == 1 ) // plugin help
		{
			PluginInterface* pluginObj = _core->pluginList()[plugin];
			QStringList output;
			output
				<< QString("Help for Plugin '%1'").arg(pluginObj->title())
				<< "----";
			if ( _core->commands[pluginObj].count() > 0 )
			{
				output << "Commands:";
				foreach(QString command, _core->commands[pluginObj].keys())
					if ( command.left(1) != "_" ) // internal comands
						output << QString("> %1.%2 : %3").arg(
							pluginObj->title()).arg(
							command, -16).arg(
							_core->commands[pluginObj][command]->description());
				output << "----";
			}
			output
				<< pluginObj->helpText()
				<< "----";
			foreach(QString line, output)
				_core->sendMessage(from, line);
			return;
		}
		else // command help
		{
			PluginInterface* pluginObj = _core->pluginList()[plugin];
			QString command = words[1].toLower();
			if ( ! _core->commands[pluginObj].contains(command) )
			{
				_core->sendMessage(from, QString("No such command: '%1'").arg(words[1]));
				return;
			}
			if ( command.left(1) == "_" )
			{
				_core->sendMessage(from, "This is an internal command.");
				return;
			}
			CommandInterface* commandObj = _core->commands[pluginObj][command];
			QStringList output;
			output
				<< QString("Help for Command '%1.%2'").arg(pluginObj->title()).arg(command)
				<< "----"
				<< "Syntax:";
			foreach(QString syntax, commandObj->syntax())
				output << QString("> %1.%2 %3").arg(
					pluginObj->title()).arg(
					command).arg(
					syntax);
			output
				<< "----"
				<< commandObj->helpText()
				<< "----";
			foreach(QString line, output)
				_core->sendMessage(from, line);
			return;
		}
	}
};
