#include "../karmacommands.h"

namespace KarmaCommands
{
	set::set(CoreInterface* core, QHash<QString,int>* scores)
	{
		_core = core;
		_scores = scores;
	}
	
	QString set::description()
	{
		return "Manually sets the karma of a word";
	}
	
	QStringList set::syntax()
	{
		return QStringList("<word> <score>");
	}
	
	QStringList set::helpText()
	{
		return QStringList("Sets the karma of the specified word to the given score.");
	}
	
	void set::go(QString to, QString from, bool pm, QString params)
	{
		QRegExp parser;
		parser.setCaseSensitivity(Qt::CaseInsensitive);
    parser.setPattern("([a-z0-9]{3,}) (-?[0-9]+)"); // try single word
 	  if ( ! parser.exactMatch(params) )
   	{
     	parser.setPattern("\"([^\"]{3,})\" (-?[0-9]+)"); // try multiple words
     	if ( !parser.exactMatch(params) )
			{
				_core->reply(to, from, pm, "Syntax: Karma.set <thing> <value>");
				return;
			}
   	}
    (*_scores)[parser.cap(1).toLower()] = parser.cap(2).toInt();

    // See if it already is in config file
    QDomElement root = _core->getPluginConfig("karma");
    for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
      if ( i.nodeName() == "item" )
      {
        QString name;
        QDomNode score;
        for ( QDomNode j = i.firstChild(); !j.isNull(); j = j.nextSibling() )
        {
          if ( j.nodeName() == "name" ) name = j.firstChild().nodeValue().toLower();
          if ( j.nodeName() == "score" ) score = j.firstChild();
        }
        if ( name != parser.cap(1) ) continue;
        else
        {
          score.setNodeValue(parser.cap(2));
          _core->saveConfig();
          _core->reply(to, from, pm, QString("Set karma of '%1' to %2.").arg(parser.cap(1),parser.cap(2)));
          return;
        }
      }

    // Need to create new entry in config file
    QDomDocument doc = _core->configDoc();

    QDomElement itemNode = doc.createElement("item");
    root.appendChild(itemNode);

      QDomElement nameNode = doc.createElement("name");
      itemNode.appendChild(nameNode);

        QDomText nameText = doc.createTextNode(parser.cap(1));
        nameNode.appendChild(nameText);

      QDomElement scoreNode = doc.createElement("score");
      itemNode.appendChild(scoreNode);

        QDomText scoreText = doc.createTextNode(parser.cap(2));
        scoreNode.appendChild(scoreText);

    _core->saveConfig();
    _core->reply(to, from, pm, QString("Set karma of '%1' to %2.").arg(parser.cap(1),parser.cap(2)));
	}
};
