#include "karma.h"
#include <iostream>
using namespace std;
extern "C" PluginInterface* getObject(CoreInterface *core)
{
	return (PluginInterface*) new Karma(core);
}

QString Karma::title()
{
	return "Karma";
}

QString Karma::description()
{
	return "Keeps a score for words";
}

QStringList Karma::helpText()
{
	return QStringList("The karma plugin keeps track of scores for words. Typing 'word++' or 'word--' adjusts the score for a given word. Quotes are also supported, such as \"slackware linux\"++.");
}

Karma::Karma(CoreInterface* core) : PluginInterface()
{
	_core = core;

	_core->registerCommand(this, "show", new KarmaCommands::show(_core, &_scores));
	_core->registerCommand(this, "set", new KarmaCommands::set(_core, &_scores));
	_core->registerCommand(this, "highScores", new KarmaCommands::highScores(_core, &_scores));
	_core->registerCommand(this, "lowScores", new KarmaCommands::lowScores(_core, &_scores));
	KarmaCommands::_messageOrAction* mOrAHandler = new KarmaCommands::_messageOrAction(_core, &_scores);
	_core->registerCommand(this, "_message", mOrAHandler);
	_core->registerCommand(this, "_action", mOrAHandler);

	// Read in current karma values from XML config file
	QDomElement root = _core->getPluginConfig("karma");
	
	for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
		if ( i.nodeName() == "item" )
		{
			QString name;
			int score;
			for ( QDomNode j = i.firstChild(); !j.isNull(); j = j.nextSibling() )
			{
				if ( j.nodeName() == "name" ) name = j.firstChild().nodeValue();
				if ( j.nodeName() == "score" ) score = j.firstChild().nodeValue().toInt();
			}
			_scores[name] = score;
		}
}
