#include "../quotescommands.h"
#include "../quote.h"

namespace QuotesCommands
{
	count::count(CoreInterface* core, QHash<unsigned int, Quote*>* quotes)
	{
			_core = core;
			_quotes = quotes;
	}

	QString count::description()
	{
		return "Counts quotes";
	}

	QStringList count::syntax()
	{
		return (QStringList()
			<< ""
			<< "<nick>"
			<< "/<pattern>/");
	}

	QStringList count::helpText()
	{
		return (QStringList()
			<< "If no parameters are given, this returns the total number of quotes. If a nickname is specified, the number of quotes containing lines said by the given nickname is returned. If a pattern is specified, the number of quotes matching the pattern is returned. For the exact syntax of patterns, see http://doc.trolltech.com/4.0/qregexp.html.");
	}
	
	void count::go(QString to, QString from, bool pm, QString params)
	{
		QRegExp parser;
		parser.setCaseSensitivity(Qt::CaseInsensitive);

		// See if we just want a total
		if ( params == "" )
		{
			_core->reply(to, from, pm, QString("There are %1 known quotes").arg(_quotes->count()));
			return;
		}

		// Check for quotedNick
		parser.setPattern("[^/ ]+");
		if ( parser.exactMatch(params) )
		{
			int count = 0;
			foreach(Quote* quote, _quotes->values())
				if ( quote->quoted().contains(params.toLower()) ) count++;
			_core->reply(to, from, pm, QString("There are %1 matching quotes").arg(count));
			return;
		}

		// Check for regex
		parser.setPattern("/(.+)/");
		if ( parser.exactMatch(params) )
		{
			int count = 0;
			parser.setPattern(parser.cap(1));
			foreach(Quote* quote, _quotes->values())
				if ( parser.indexIn(quote->quoteLines().join("\n")) != -1 )
					count++;
			_core->reply(to, from, pm, QString("There are %1 matching quotes").arg(count));
			return;
		}
		_core->reply(to, from, pm, "Invalid syntax, see Help.Help Quotes.Count");
	}
};
