#include "../quotescommands.h"
#include "../quote.h"

namespace QuotesCommands
{
	good::good(CoreInterface* core, QHash<unsigned int, Quote*>* quotes, Quote** lastQuote, unsigned int* highestQuote)
	{
			_core = core;
			_quotes = quotes;
			_lastQuote = lastQuote;
			_highestQuote = highestQuote;
	}

	QString good::description()
	{
		return "Increases score of a quote";
	}

	QStringList good::helpText()
	{
		return QStringList("If no parameters are specified, the score of the last displayed quote is increased; otherwise the score of the specified quote is increased.");
	}
	
	QStringList good::syntax()
	{
		return (QStringList()
			<< ""
			<< "<number>");
	}

	void good::go(QString to, QString from, bool pm, QString params)
	{
		QRegExp parser;
		parser.setCaseSensitivity(Qt::CaseInsensitive);
		// See if this is lastQuote
		if ( params == "" )
		{
			LASTQUOTE_CHECK;
			unsigned int quoteNum = _quotes->key(*_lastQuote);
			(*_lastQuote)->incrementScore();
			saveQuote(quoteNum, _core, _quotes, _highestQuote);
			_core->reply(to, from, pm, QString("Given karma to quote #%1; new karma is %2").arg(
				quoteNum).arg((*_lastQuote)->quoteScore()));
			return;
		}

		// Must be quote-by-number
		parser.setPattern("([0-9]+)");
		if ( parser.exactMatch(params) )
		{
			unsigned int quoteNum = parser.cap(1).toUInt();
			if ( ! _quotes->contains(quoteNum) )
			{
				_core->reply(to, from, pm, "No such quote");
				return;
			}
			*_lastQuote = (*_quotes)[quoteNum];
			(*_lastQuote)->incrementScore();
			saveQuote(quoteNum, _core, _quotes, _highestQuote);
			_core->reply(to, from, pm, QString("Given karma to quote #%1; new karma is %2").arg(
				quoteNum,
				(*_lastQuote)->quoteScore()));
			return;
		}
		
		_core->reply(to, from, pm, "Invalid syntax, see Help.Help Quotes.Good");
	}
};
