#include "../quotescommands.h"
#include "../quote.h"

namespace QuotesCommands
{
	remove::remove(CoreInterface* core, QHash<unsigned int, Quote*>* quotes, Quote** lastQuote)
	{
			_core = core;
			_quotes = quotes;
			_lastQuote = lastQuote;
	}

	QString remove::description()
	{
		return "Removes a quote from the database";
	}

	QStringList remove::helpText()
	{
		return QStringList("Removes a quote from the database. If no parameters are given, the last quote is removed. If a number is specified, the quoteswith that number is removed.");
	}
	
	QStringList remove::syntax()
	{
		return (QStringList()
			<< ""
			<< "<number>");
	}

	void remove::go(QString to, QString from, bool pm, QString params)
	{
		QRegExp parser;
		parser.setCaseSensitivity(Qt::CaseInsensitive);
		// See if this is lastQuote
		if ( params == "" )
		{
			LASTQUOTE_CHECK;
			unsigned int quoteNum = _quotes->key(*_lastQuote);
			removeQuote(quoteNum, _core, _quotes, _lastQuote);
			_core->reply(to, from, pm, QString("Deleted quote #%1").arg(quoteNum));
			return;
		}

		// Else, must be number
		parser.setPattern("([0-9]+)");
		if ( parser.exactMatch(params) )
		{
			unsigned int quoteNum = parser.cap(1).toUInt();
			if ( ! _quotes->contains(quoteNum) )
			{
				_core->reply(to, from, pm, "No such quote");
				return;
			}
			removeQuote(quoteNum, _core, _quotes, _lastQuote);
			_core->reply(to, from, pm, QString("Deleted quote #%1").arg(quoteNum));
			return;
		}
		
		_core->reply(to, from, pm, "Invalid syntax, see Help.Help Quotes.Remove");
	}
};
