#include "quotescommands.h"

#include <QString>

namespace QuotesCommands
{
	void outputQuote(QString to, QString from, bool pm, unsigned int quoteNum, QHash<unsigned int, Quote*>* quotes, CoreInterface* core)
	{
		Quote* quote = (*quotes)[quoteNum];
		core->reply(to, from, pm, QString("Quote #%1, quoted by '%2' at %3 with score of %4:"
			).arg(QString::number(quoteNum)
			).arg(quote->quotedBy()
			).arg(quote->quoteTime()
			).arg(QString::number(quote->quoteScore())));
		foreach(QString line, quote->quoteLines())
			core->reply(to, from, pm, line);
	}

	QString getConfig(CoreInterface* core, QString key, QString defaultValue)
	{
		QDomNode root = core->getPluginConfig("quotes");
		for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
			if ( i.nodeName() == "config" )
			{
				QDomElement configElement = i.toElement();
				if ( i.toElement().attribute("key") == key )
					return i.toElement().attribute("value");
			}
		saveConfig(core, key, defaultValue);
		return defaultValue;
	}

	void saveConfig(CoreInterface* core, QString key, QString value)
	{
		QDomDocument doc = core->configDoc();
		QDomElement root = core->getPluginConfig("quotes");
		
		// See if this config key is already set
		for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
			if ( i.nodeName() == "config" )
			{
				QDomElement configElement = i.toElement();
				if ( i.toElement().attribute("key") == key )
				{
					i.toElement().setAttribute("value", value);
					return;
				}
			}
			
		// New config key	needs creating
		QDomElement configElement = doc.createElement("config");
		configElement.setAttribute("key", key);
		configElement.setAttribute("value", value);
		root.appendChild(configElement);
		core->saveConfig();
	}

	void saveQuote(unsigned int quoteNum, CoreInterface* core, QHash<unsigned int, Quote*>* quotes, unsigned int *highestQuote)
	{
		QDomDocument doc = core->configDoc();
		QDomElement root = core->getPluginConfig("quotes");

		for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
			if ( i.nodeName() == "quote" && i.toElement().attribute("number") == QString::number(quoteNum) )
			{
				for ( QDomNode j = i.firstChild(); !j.isNull(); j = j.nextSibling() ) // only thing that could have changed is score
					if ( j.nodeName() == "score" ) j.firstChild().setNodeValue(QString::number((*quotes)[quoteNum]->quoteScore()));
				saveConfig(core, "highestQuote", QString::number(*highestQuote));
				core->saveConfig();
				return;
			}

		// If we get here, this is a new quote
		QDomElement quoteNode = doc.createElement("quote");
		quoteNode.setAttribute("number", QString::number(quoteNum));
		root.appendChild(quoteNode);

		QDomElement dataNode = doc.createElement("data");
		quoteNode.appendChild(dataNode);

		QDomCDATASection dataCDATA = doc.createCDATASection((*quotes)[quoteNum]->quoteLines().join("\n"));
		dataNode.appendChild(dataCDATA);

		QDomElement quoterNode = doc.createElement("quotedBy");
		quoteNode.appendChild(quoterNode);

		QDomText quoterText = doc.createTextNode((*quotes)[quoteNum]->quotedBy());
		quoterNode.appendChild(quoterText);

		QDomElement scoreNode = doc.createElement("score");
		quoteNode.appendChild(scoreNode);

		QDomText scoreText = doc.createTextNode(QString::number((*quotes)[quoteNum]->quoteScore()));
		scoreNode.appendChild(scoreText);

		QDomElement timeNode = doc.createElement("time");
		quoteNode.appendChild(timeNode);

		QDomText timeText = doc.createTextNode((*quotes)[quoteNum]->quoteTime());
		timeNode.appendChild(timeText);

		QDomElement quotedNode = doc.createElement("quoted");
		quoteNode.appendChild(quotedNode);

		foreach(QString nick, (*quotes)[quoteNum]->quoted())
		{
			QDomElement nickNode = doc.createElement("nick");
			quotedNode.appendChild(nickNode);

			QDomText nickText = doc.createTextNode(nick);
			nickNode.appendChild(nickText);
		}

		saveConfig(core, "highestQuote", QString::number(quoteNum));
		*highestQuote = quoteNum;
		core->saveConfig();
	}

	void removeQuote(unsigned int quoteNum, CoreInterface* core, QHash<unsigned int, Quote*>* quotes, Quote** lastQuote)
	{
		*lastQuote = 0;
		quotes->remove(quoteNum);
		
		// Remove from the XML config file
		QDomElement root = core->getPluginConfig("quotes");
		for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
			if ( i.nodeName() == "quote" && i.toElement().attribute("number") == QString::number(quoteNum) )
			{
				root.removeChild(i);
				break;
			}
		core->saveConfig();
	}
};
