#include "quotes.h"
#include "quotescommands.h"
#include <stdlib.h>

#include <QDateTime>

extern "C" PluginInterface* getObject(CoreInterface *core)
{
	return (PluginInterface*) new Quotes(core);
}

QStringList Quotes::helpText()
{
	return QStringList("The quotes plugin stores, retrieves, and keeps a score for quotes.");
}

Quotes::Quotes(CoreInterface *core)
{
	_core = core;
	_lastQuote = 0;
	_highestQuote = QuotesCommands::getConfig(_core, "highestQuote", "0").toInt();
	QuotesCommands::getConfig(_core, "logLength", "30"); // initialise default value
	srand(time(NULL));

	_core->registerCommand(this, "count", new QuotesCommands::count(_core, &_quotes));
	_core->registerCommand(this, "bad", new QuotesCommands::bad(_core, &_quotes, &_lastQuote, &_highestQuote));
	_core->registerCommand(this, "good", new QuotesCommands::good(_core, &_quotes, &_lastQuote, &_highestQuote));
	_core->registerCommand(this, "remove", new QuotesCommands::remove(_core, &_quotes, &_lastQuote));
	_core->registerCommand(this, "get", new QuotesCommands::get(_core, &_quotes, &_lastQuote));
	_core->registerCommand(this, "add", new QuotesCommands::add(_core, &_quotes, &_lastQuote, &_log, &_highestQuote));
	_core->registerCommand(this, "_message", new QuotesCommands::_message(_core, &_log));
	_core->registerCommand(this, "_action", new QuotesCommands::_action(_core, &_log));
	
	QDomElement root = _core->getPluginConfig("quotes");
	for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
		if ( i.nodeName() == "quote" )
		{
			Quote* quote = new Quote;
			_quotes[i.toElement().attribute("number").toInt()] = quote;
			
			for ( QDomNode j = i.toElement().firstChild(); !j.isNull(); j = j.nextSibling() )
			{
				if ( j.nodeName() == "data" )
					foreach(QString line, j.firstChild().nodeValue().split(" "))
						quote->addLine(line);
				if ( j.nodeName() == "quotedBy" )
					quote->setQuotedBy(j.firstChild().nodeValue());
				if ( j.nodeName() == "score" )
					quote->setScore(j.firstChild().nodeValue().toInt());
				if ( j.nodeName() == "time" )
					quote->setTime(j.firstChild().nodeValue());
				if ( j.nodeName() == "quoted" )
					for ( QDomNode k = j.firstChild(); !k.isNull(); k = k.nextSibling() )
						if ( k.nodeName() == "nick" )
							quote->addQuotedNick(k.firstChild().nodeValue());
			}
		}
}

QString Quotes::title()
{
	return "Quotes";
}

QString Quotes::description()
{
	return "Remembers and scores quotes";
}

Quotes::~Quotes()
{
	foreach(Quote* quote, _quotes)
		delete quote;
}
