#include "../securitycommands.h"

namespace SecurityCommands
{
	revoke::revoke(CoreInterface* core, QList<QStringList>* rulesTable)
	{
		_core = core;
		_rulesTable = rulesTable;
	}
	
	QString revoke::description()
	{
		return "Removes a permissions rule";
	}
	
	QStringList revoke::syntax()
	{
		return QStringList("<user> <plugin> <command>");
	}
	
	QStringList revoke::helpText()
	{
		return QStringList("Removes a rule; see Security.list for a list of current rules.");
	}
	
	void revoke::go(QString to, QString from, bool pm, QString params)
	{
		QStringList words = params.split(" ");
		if ( words.count() != 3 )
		{
			_core->reply(to, from, pm, "Syntax: Security.revoke <user> <plugin> <command>");
			return;
		}

		QDomElement root = _core->getPluginConfig("security");
		for ( QDomNode i = root.firstChild(); !i.isNull(); i = i.nextSibling() )
		{
			QString ruleUser = "", rulePlugin = "", ruleCommand = "";
			for ( QDomNode j = i.firstChild(); !j.isNull(); j = j.nextSibling() )
			{
				if ( j.nodeName() == "user" ) ruleUser = j.firstChild().nodeValue().toLower();
				if ( j.nodeName() == "plugin" ) rulePlugin = j.firstChild().nodeValue().toLower();
				if ( j.nodeName() == "command" ) ruleCommand = j.firstChild().nodeValue().toLower();
			}
			if ( ruleUser == words[0].toLower() && rulePlugin == words[1].toLower() && ruleCommand == words[2].toLower() )
			{
				root.removeChild(i);
				_core->saveConfig();
				for ( int i = 0; i < _rulesTable->size(); i++)
				{
					QStringList rule = _rulesTable->at(i);
					if ( rule[0] == words[0].toLower() && rule[1] == words[1].toLower() && rule[2] == words[2].toLower() )
					{
						_rulesTable->removeAt(i);
						break;
					}
				}
				_core->reply(to, from, pm, "Revoked rule.");
				return;
			}
		}
		_core->reply(to, from, pm, "Rule not found.");
		return;
	}
};
