#include "../../include/commandinterface.h"
#include "../../include/coreinterface.h"

#include <QString>
#include <QStringList>

#define GRANT(user, plugin, command)\
{\
	QDomDocument doc = _core->configDoc(); \
	QDomElement root = _core->getPluginConfig("security"); \
	QDomElement ruleNode = doc.createElement("rule"); \
	root.appendChild(ruleNode); \
		QDomElement userNode = doc.createElement("user"); \
		ruleNode.appendChild(userNode); \
			QDomText userText = doc.createTextNode(user); \
			userNode.appendChild(userText); \
		QDomElement pluginNode = doc.createElement("plugin"); \
		ruleNode.appendChild(pluginNode); \
			QDomText pluginText = doc.createTextNode(plugin); \
			pluginNode.appendChild(pluginText); \
		QDomElement commandNode = doc.createElement("command"); \
		ruleNode.appendChild(commandNode); \
			QDomText commandText = doc.createTextNode(command); \
			commandNode.appendChild(commandText); \
	_core->saveConfig(); \
	_rulesTable->append(QStringList() << user << plugin << command); \
};

namespace SecurityCommands
{
	class auth : public CommandInterface
	{
	private:
		CoreInterface* _core;
	public:
		auth(CoreInterface* core);
		virtual QString description();
		virtual QStringList syntax();
		virtual QStringList helpText();
		virtual void go(QString to, QString from, bool pm, QString params);
	};

	class _raw : public CommandInterface
	{
	private:
		CoreInterface* _core;
		QStringList* _authedNicks;
	public:
		_raw(CoreInterface* core, QStringList* authedNicks);
		virtual void go(QString to, QString from, bool pm, QString params);
	};

	class list : public CommandInterface
	{
	private:
		CoreInterface* _core;
	public:
		list(CoreInterface* core);
		virtual QString description();
		virtual QStringList syntax();
		virtual QStringList helpText();
		virtual void go(QString to, QString from, bool pm, QString params);
	};
	
	class grant : public CommandInterface
	{
	private:
		CoreInterface* _core;
		QList<QStringList>* _rulesTable;
	public:
		grant(CoreInterface* core, QList<QStringList>* rulesTable);
		virtual QString description();
		virtual QStringList syntax();
		virtual QStringList helpText();
		virtual void go(QString to, QString from, bool pm, QString params);
	};

	class revoke : public CommandInterface
	{
	private:
		CoreInterface* _core;
		QList<QStringList>* _rulesTable;
	public:
		revoke(CoreInterface* core, QList<QStringList>* rulesTable);
		virtual QString description();
		virtual QStringList syntax();
		virtual QStringList helpText();
		virtual void go(QString to, QString from, bool pm, QString params);
	};

	class _userQuit : public CommandInterface
	{
	private:
		CoreInterface* _core;
		QStringList* _authedNicks;
	public:
		_userQuit(CoreInterface* core, QStringList* authedNicks);
		virtual void go(QString to, QString from, bool pm, QString params);
	};
};
