#include "../tellscommands.h"

namespace TellsCommands
{
	tell::tell(CoreInterface* core, QList<tellInfo>* tells)
	{
		_core = core;
		_tells = tells;
	}
	
	QString tell::description()
	{
		return "Sends a user a message";
	}
	
	QStringList tell::syntax()
	{
		return QStringList("<user> <message>");
	}
	
	QStringList tell::helpText()
	{
		return QStringList("Sends the user named the specified message.");
	}
	
	void tell::go(QString to, QString from, bool pm, QString params)
	{
		QStringList words = params.split(" ");
		if ( words.count() < 2 )
		{
			_core->reply(to, from, pm, "Syntax: Tells.tell <target> <message>");
			return;
		}

		QString target = words.takeFirst().toLower();
		QString time = QDateTime::currentDateTime().toString();
		QString message = words.join(" ");

		QDomDocument doc = _core->configDoc();
		QDomElement root = _core->getPluginConfig("tells");

		QDomElement tellNode = doc.createElement("tell");
		root.appendChild(tellNode);

		QDomElement toNode = doc.createElement("to");
		tellNode.appendChild(toNode);

		QDomText toText = doc.createTextNode(target);
		toNode.appendChild(toText);

		QDomElement fromNode = doc.createElement("from");
		tellNode.appendChild(fromNode);

		QDomText fromText = doc.createTextNode(from);
		fromNode.appendChild(fromText);

		QDomElement timeNode = doc.createElement("time");
		tellNode.appendChild(timeNode);

		QDomText timeText = doc.createTextNode(time);
		timeNode.appendChild(timeText);

		QDomElement messageNode = doc.createElement("message");
		tellNode.appendChild(messageNode);

		QDomText messageText = doc.createTextNode(message);
		messageNode.appendChild(messageText);
		_core->saveConfig();

		tellInfo tell;
		tell[TellTo] = target;
		tell[TellFrom] = from;
		tell[TellTime] = time;
		tell[TellMessage] = message;
		_tells->append(tell);

		_core->reply(to, from, pm, "Okay, will tell upon next speaking.");
	}
};
